import 'dart:convert';

import 'package:flutter/cupertino.dart';
import 'package:tim_ui_kit_lbs_plugin/i18n/strings.g.dart';

class I18nUtils {
  static I18nUtils? _instance;
  I18nUtils._internal(BuildContext context) {
    _init(context);
  }
  factory I18nUtils(BuildContext context) {
    _instance ??= I18nUtils._internal(context);
    return _instance!;
  }

  Map<String, dynamic> zhMap = {};
  Map zhMapRevert = {};
  RegExp expForParameterOut = RegExp(r"{{[^]+}}");
  RegExp expForParameter = RegExp(r"(?<=\{{)[^}]*(?=\}})");
  late final t;

  void _init(BuildContext context) {
    try {
      t = getI18NBuild(context);
    } catch (e) {
      t = AppLocale.zh.build();
    }
    zhMap = jsonDecode(zhJson);
    zhMapRevert = revertMap(zhMap);
  }

  String imt(String value) {
    String currentKey = zhMapRevert[value] ?? getKeyFromMap(zhMap, value) ?? "";
    String translatedValue = t[currentKey] ?? value;
    return translatedValue;
  }

  Function imt_para(String template, String value) {
    // 调用模板：imt_para("已选：{{addType}}",'已选：$addType')(addType: addType)
    final originTemplate = template.replaceAllMapped(
        expForParameterOut, (Match m) => replaceParameterForTemplate(m));
    final originKey = zhMapRevert[originTemplate] ??
        getKeyFromMap(zhMap, originTemplate) ??
        "";
    final Function translatedValueFunction = t[originKey] ?? textErrorFunction;
    return translatedValueFunction;
  }

  String replaceParameterForTemplate(Match value) {
    final String? parameter = expForParameter.stringMatch(value[0] ?? "");
    return "\$$parameter";
  }

  String textErrorFunction() {
    return "文本错误";
  }

  static String getKeyFromMap(Map map, String key) {
    String currentKey = "";
    for (String tempKey in map.keys) {
      if (map[tempKey] == key) {
        currentKey = tempKey;
        break;
      }
    }
    return currentKey;
  }

  static Map revertMap(Map map) {
    final Map<String, String> newMap = {};
    for (String tempKey in map.keys) {
      newMap[map[tempKey]] = tempKey;
    }
    return newMap;
  }

  String getCurrentLanguage(BuildContext context) {
    return Localizations.localeOf(context).languageCode;
  }

  Map i18nLanguageMap = {
    "en": AppLocale.en.build(),
    "zh": AppLocale.zh.build(),
  };

  getI18NBuild(BuildContext context) {
    return i18nLanguageMap[getCurrentLanguage(context)] ?? AppLocale.en.build();
  }

  final zhJson =
      '''{"k_0b9q00i":"腾讯大厦/////深圳市南山区深南大道10000号","k_03fw6h1":"深圳市","k_0f7mdn7":"检索失败","k_003nevv":"取消","k_002r305":"发送","k_158cssa":"腾讯地图","k_157htdw":"高德地图","k_157im4z":"百度地图","k_157m20e":"苹果地图","k_1bqk7kl":"未检测到高德地图","k_12ba1nv":"未检测到腾讯地图","k_1dz8sci":"未检测到百度地图","k_0f7fvky":"打开失败","k_1l2v0we":"腾讯大厦内，招行信息研发大厦附近18米","k_10o8gk2":"搜索地点","k_10o86v7":"未知地点"}''';
}
